water_lod_control <- SpawnEntityFromTable("water_lod_control", {});

PrecacheModel("models/vip_mobster/hud/vip_compass.mdl")

function remap(value, in_min, in_max, out_min, out_max)
{
    return out_min + (((value - in_min) * (out_max - out_min)) / (in_max - in_min));
}

::compassHelpers <- player_table();
::lastObserverMode <- player_table();

OnGameEvent("player_spawn_post", function(player, args)
{
    player.SetScriptOverlayMaterial("vip_mobster/hud/vip_compass");
    if (player in compassHelpers && compassHelpers[player] && IsValid(compassHelpers[player]))
        UpdateCompassAttachmentMode(player);
    else
        CreateCompassHelper(player);
});

AddTimer(-1, function()
{
    foreach(player in GetPlayers())
    {
        if (GetPropInt(player, "m_iObserverMode") > 1)
        {
            if (!(player in lastObserverMode) || lastObserverMode[player] < 1)
            {
                if (player in compassHelpers && compassHelpers[player] && IsValid(compassHelpers[player]))
                    UpdateCompassAttachmentMode(player);
                lastObserverMode[player] <- 1;
            }
        }
        else
            lastObserverMode[player] <- 0;
    }
});

function CreateCompassHelper(player)
{
    local viewmodel = GetPropEntity(player, "m_hViewModel");
    if (!viewmodel)
        return;

    local newCompassHelper = CreateByClassname("obj_teleporter");
    newCompassHelper.KeyValueFromString("targetname", "helper");
    newCompassHelper.DispatchSpawn();
    newCompassHelper.SetModel("models/vip_mobster/hud/vip_compass.mdl");
    newCompassHelper.AddEFlags(EFL_NO_THINK_FUNCTION);
    newCompassHelper.SetSolid(SOLID_NONE);
    SetPropBool(newCompassHelper, "m_bPlacing", true);
    SetPropInt(newCompassHelper, "m_fObjectFlags", 2);
    SetPropEntity(newCompassHelper, "m_hBuilder", player);
    SetPropInt(newCompassHelper, "m_clrRender", 0xFFFFFFFF);

    SetPropInt(newCompassHelper, "m_fEffects", 145);
    SetPropInt(newCompassHelper, "m_nNextThinkTick", 0x7FFFFFFF);
    newCompassHelper.AcceptInput("DisableShadow", null, null, null);

    compassHelpers[player] <- newCompassHelper;
    UpdateCompassAttachmentMode(player);
}

function UpdateCompassAttachmentMode(player)
{
    local compassHelper = compassHelpers[player];
    if (!player.IsAlive() /*|| GetRoundState() == GR_STATE_TEAM_WIN*/)
    {
        compassHelper.AcceptInput("SetParent", "", null, null);
        compassHelper.SetAbsOrigin(Vector(0, 0, 0));
    }
    else if (IsVIP(player))
    {
        compassHelper.AcceptInput("SetParent", "", null, null);
        compassHelper.SetAbsOrigin(Vector(0, 0, 1));
    }
    else
    {
        local viewmodel = GetPropEntity(player, "m_hViewModel");
        if (!viewmodel)
            return;
        compassHelper.SetAbsOrigin(viewmodel.GetOrigin() + player.EyeAngles().Forward() * 200);
        compassHelper.AcceptInput("SetParent", "!activator", viewmodel, viewmodel);
    }
}

function UpdateCompassData()
{
    local target = FindVIP(TF_TEAM_BLUE);
    if (target && target.IsAlive())
    {
        SetPropFloat(water_lod_control, "m_flCheapWaterStartDistance", target.GetOrigin().x);
        SetPropFloat(water_lod_control, "m_flCheapWaterEndDistance", target.GetOrigin().y);
        local remapped = 1 - remap(target.GetHealth() / target.GetMaxHealth().tofloat(), 0.0, 1.0, 0.09, 0.88);
        SetPropVector(worldspawn, "m_WorldMins", Vector(remapped, 0, 0));
    }
    else
    {
        SetPropVector(worldspawn, "m_WorldMins", Vector(1.1, 0, 0));
    }
}
AddTimer(-1, UpdateCompassData);