::stv_player <- null;
::client_cache <- [];
::player_cache <- [ [], [], [], [] ];
::alive_player_cache <- [ [], [], [], [] ];

RunWithDelay(1, function()
{
    if (!stv_player)
        stv_player = tf_gamerules;
});

//A valid _client_ can be a spectator. A valid _player_ can not.
::GetClients <- function()
{
    return client_cache;
}
::GetAllClients <- GetClients

//A valid _client_ can be a spectator. A valid _player_ can not.
::GetPlayers <- function(team = 0)
{
    return player_cache[team];
}
::GetAllPlayers <- GetPlayers

::GetAlivePlayers <- function(team = 0)
{
    return alive_player_cache[team];
}

::CTFPlayer.IsAlive <- function()
{
    return GetPropInt(this, "m_lifeState") == 0;
}
::CTFBot.IsAlive <- CTFPlayer.IsAlive;

::CTFPlayer.IsOnGround <- function()
{
    return GetPropEntity(this, "m_hGroundEntity") != null;
}
::CTFBot.IsOnGround <- CTFPlayer.IsOnGround;

//A valid _client_ can be a spectator. A valid _player_ can not.
::IsValidClient <- function(player)
{
    try
    {
        return player && player.IsValid() && player.IsPlayer();
    }
    catch(e)
    {
        return false;
    }
}

//A valid _client_ can be a spectator. A valid _player_ can not.
::IsValidPlayer <- function(player)
{
    try
    {
        return player && player.IsValid() && player.IsPlayer() && player.GetTeam() > 1;
    }
    catch(e)
    {
        return false;
    }
}

::IsValidPlayerOrBuilding <- function(entity)
{
    try
    {
        return entity
            && entity.IsValid()
            && entity.GetTeam() > 1
            && (entity.IsPlayer() || startswith(entity.GetClassname(), "obj_"));
    }
    catch(e)
    {
        return false;
    }
}

::IsValidBuilding <- function(building)
{
    try
    {
        return building
            && building.IsValid()
            && startswith(building.GetClassname(), "obj_")
            && building.GetTeam() > 1;
    }
    catch(e)
    {
        return false;
    }
}

::CTFPlayer.GetUserID <- function()
{
    return GetPropIntArray(tf_player_manager, "m_iUserID", this.entindex());
}
::CTFBot.GetUserID <- CTFPlayer.GetUserID;

::GetPlayerFromParams <- function(params, key = "userid")
{
    if (!(key in params))
        return null;
    local player = GetPlayerFromUserID(params[key]);
    if (IsValidPlayer(player))
        return player;
    return null;
}

::CTFPlayer.Yeet <- function(vector)
{
    SetPropEntity(this, "m_hGroundEntity", null);
    this.ApplyAbsVelocityImpulse(vector);
    this.RemoveFlag(FL_ONGROUND);
}
::CTFBot.Yeet <- CTFPlayer.Yeet;

//Normal ForceChangeTeam will not work if the player is dueling. This is a fix.
::CTFBot.SwitchTeam <- function(team)
{
    this.ForceChangeTeam(team, true);
    SetPropInt(this, "m_iTeamNum", team);
}

::CTFPlayer.SwitchTeam <- function(team)
{
    SetPropInt(this, "m_bIsCoaching", 1);
    this.ForceChangeTeam(team, true);
    SetPropInt(this, "m_bIsCoaching", 0);
}

::CTFPlayer.GetWeaponsAndCosmetics <- function()
{
    local items = [];
    local extraVM;
    for (local item = this.FirstMoveChild(); item != null; item = item.NextMovePeer())
    {
        local className = item.GetClassname();
        if (item && item.IsValid() && (startswith(className, "tf_we") || className == "tf_powerup_bottle"))
        {
            items.push(item);
            extraVM = GetPropEntity(item, "m_hExtraWearableViewModel");
            if (extraVM && extraVM.IsValid())
                items.push(extraVM);
        }
    }
    return items;
}
::CTFBot.GetWeaponsAndCosmetics <- CTFPlayer.GetWeaponsAndCosmetics;

::CTFPlayer.GetAttachments <- function()
{
    local items = [];
    local extraVM;
    for (local item = this.FirstMoveChild(); item != null; item = item.NextMovePeer())
        if (item && item.IsValid() && item.GetClassname() != "tf_viewmodel")
        {
            items.push(item);
            extraVM = GetPropEntity(item, "m_hExtraWearableViewModel");
            if (extraVM && extraVM.IsValid())
                items.push(extraVM);
        }
    return items;
}
::CTFBot.GetAttachments <- CTFPlayer.GetAttachments;

::CTFPlayer.GetWeaponBySlot <- function(slot)
{
	for (local i = 0; i < 27; i++)
	{
		local weapon = GetPropEntityArray(this, "m_hMyWeapons", i);
        if (weapon && weapon.IsValid() && weapon.GetSlot() == slot)
            return weapon;
	}
    return null;
}
::CTFBot.GetWeaponBySlot <- CTFPlayer.GetWeaponBySlot;

::CTFPlayer.Heal <- function(healing, dispalyOnHud = true)
{
    local oldHP = this.GetHealth();
    local newHP = clampFloor(oldHP, clampCeiling(oldHP + healing, this.GetMaxHealth()));
    this.SetHealth(newHP);
    if (dispalyOnHud && newHP - oldHP > 0)
        SendGlobalGameEvent("player_healonhit", {
            entindex = this.entindex(),
            amount = newHP - oldHP
        });
}
::CTFBot.Heal <- CTFPlayer.Heal;

//Might change in the future.
//I dislike that you need to manually type `player.entindex()` as a key,
// but using the ent index is the whole point - it gives us safety against nulls, invalid entities and out-of-bounds
::player_collection <- function(defValue = 0)
{
    local array = [];
    array.resize(MAX_PLAYERS + 1, defValue);
    OnGameEvent("player_connect", -999, function(player, params)
    {
        this[params.index+1] = defValue;
    }, array);
    return array;
}
::playerCollection <- player_collection;
::PlayerCollection <- player_collection;
::GetPlayerCollection <- player_collection;