Include("_charlib/weapon_list.nut");
Include("_charlib/custom_sounds.nut");

::characterAddonScripts <- [];
::customCharacterClasses <- CACHE_CUSTOM_CHARACTERS ? GetPersistentVar("customCharacterClasses", {}) : {};

function IncludeCharacter(characterName, characterScripts, func = null)
{
    if (characterName in customCharacterClasses)
        return customCharacterClasses[characterName];
    local scope = {};
    Include("_charlib/custom_character.nut", scope);
    if (func)
        func.call(scope);
    foreach (characterScript in characterScripts)
        IncludeScript(characterScript, scope);
    foreach (characterAddonScript in characterAddonScripts)
        try { IncludeScript.call(scope, characterAddonScript); } catch(e) { }

    delete scope.RunWithDelay;
    delete scope.OnGameEvent;
    delete scope.AddTimer;

    local characterClass = class
    {
        self = null;
        constructor(player)
        {
            self = player;
            Init();
        }
    };
    foreach(k, v in scope)
        characterClass[k] <- v;
    characterClass.name <- characterName;
    customCharacterClasses[characterName] <- characterClass;
}

::ConvertToCustomCharacter <- function(player, characterName)
{
    ClearCustomCharacter(player);
    player.ValidateScriptScope();
    player.GetScriptScope().customCharacter <- customCharacterClasses[characterName](player);
}

::ClearCustomCharacter <- function(player)
{
    local customCharacter = GetCustomCharacter(player);
    if (customCharacter)
    {
        customCharacter.ClearCustomCharacterInternal.call(customCharacter);
        return true;
    }
    return false;
}

::GetCustomCharacter <- function(player)
{
    return safescriptget(player, "customCharacter", null);
}