myInits <- [];
myEvents <- [];
myTimers <- [];
myDelays <- [];

OnInit <- function(func)
{
    myInits.push(func);
}

OnGameEvent <- function(eventName, order, listenerFunc = null, scope = null, selfListener = false)
{
    myEvents.push([eventName, order, listenerFunc, scope, selfListener]);
}

AddTimer <- function(interval, order, ...)
{
    myTimers.push([interval, order].extend(vargv));
}

RunWithDelay <- function(delay, delayedFunc, ...)
{
    myDelays.push([delay, delayedFunc].extend(vargv));
}

playerClass <- null;
model <- null;
viewmodel <- null;
scale <- 1.0;
deleteItems <- false;
mute <- false;
cleanOnDeath <- false;
cleanOnClassChange <- false;
cleanAttachmentsOnDeath <- true;

extraData <- {};
extraAttachments <- [];
startingClass <- 1;
antiLoopFlag <- true;

function Init()
{
    foreach (init in myInits)
        init();
    foreach (event in myEvents)
        OnGameEvent(event[0], event[1], event[2], event[3], event[4]);
    foreach (timer in myTimers)
        AddTimer.acall([this].extend(timer));
    foreach (delay in myDelays)
        RunWithDelay.acall([this].extend(delay));

    ConvertToCharacterInternal();
}

function ConvertToCharacterInternal()
{
    if (playerClass)
        self.SetPlayerClass(playerClass);
    startingClass = self.GetPlayerClass();
    self.Regenerate(true);
}

OnSelfEvent("post_inventory_application", -20, function(player, params)
{
    if (antiLoopFlag = !antiLoopFlag)
        return EVENT_CANCEL;
});

OnSelfEvent("post_inventory_application", -19, function(params)
{
    if (cleanOnClassChange && self.GetPlayerClass() != startingClass)
    {
        ClearCustomCharacterInternal(true);
        return EVENT_CANCEL;
    }

    if (playerClass)
    {
        self.SetPlayerClass(playerClass);
        self.Regenerate(true);
    }
    else
        antiLoopFlag = !antiLoopFlag;

    if (deleteItems)
        foreach(item in self.GetAttachments())
            KillIfValid(item);
});

OnSelfEvent("post_inventory_application_next", -18, function(params)
{
    self.SetCustomModelWithClassAnimations(model);
    self.SetModelScale(scale, 0);

    if (mute)
        self.AddCustomAttribute("voice pitch scale", 0, -1);

    FireCustomEvent("init_custom_character", params);
});

function ClearCustomCharacterInternal(respawn = false)
{
    try
    {
        FireCustomEvent("clear_custom_character", {
        	userid = self.GetUserID()
        });
    }
    catch(e) { }

    if (cleanAttachmentsOnDeath)
    {
        foreach(item in self.GetAttachments())
            if (IsValid(item) && item.GetClassname() != "tf_weapon_invis" && item.GetClassname() != "tf_weapon_medigun") //todo these weapons have something odd about them
                KillIfValid(item);
        foreach(ent in extraAttachments)
            KillIfValid(ent);
    }

    if (!model)
        self.SetCustomModelWithClassAnimations("");

    if (respawn == true)
        RunWithDelay(0.01, function(player) {
            if (IsValidPlayer(player))
            {
                player.ForceRegenerateAndRespawn();
                player.SetHealth(player.GetMaxHealth());
            }
        }, self, main_script);

    delete self.GetScriptScope().customCharacter;
}

OnSelfEvent("player_death", 0, function()
{
    RunWithDelay(0.5, function()
    {
        if (cleanOnDeath)
            ClearCustomCharacterInternal();
    });
});

OnSelfEvent("player_disconnect", 0, ClearCustomCharacterInternal);
OnSelfEvent("player_team", 0, ClearCustomCharacterInternal);
OnGameEvent("scorestats_accumulated_update", 0, ClearCustomCharacterInternal);

Include("_charlib/weapons.nut");
Include("_charlib/weapon_models.nut");
Include("_charlib/sound_bank.nut");
