class PlaySoundBuilderClass
{
    sound = null;
    scope = null;
    entity = null;
    delay = null;
    isGlobal = null;
    positional = null;
    voice = null;
    channel = null;
    paramsOverride = null;
    soundLevel = 150;
    volume = 1;
    singlePlayer = null;
    double = null;

    constructor(sound)
    {
        this.sound = sound;
        return this;
    }

    function Scope(scopeOrEnt)
    {
        if ("IsValid" in scopeOrEnt)
        {
            scopeOrEnt.ValidateScriptScope();
            this.entity = scopeOrEnt;
            this.scope = scopeOrEnt.GetScriptScope();
            if (this.scope)
                this.scope = this.scope.weakref();
        }
        else
        {
            entity = safeget(scopeOrEnt, "self", null);
            this.scope = scopeOrEnt;
            if (this.scope)
                this.scope = this.scope.weakref();
        }
        return this;
    }

    function Delay(delay)
    {
        this.delay = delay;
        return this;
    }

    function Entity(entity)
    {
        this.entity = entity;
        return this;
    }

    function Global(isGlobal = true)
    {
        this.isGlobal = isGlobal;
        return this;
    }

    function Positional(positional = true)
    {
        this.positional = positional;
        return this;
    }

    function SoundLevel(soundLevel)
    {
        this.soundLevel = soundLevel;
        return this;
    }

    function Voice(voice = true)
    {
        this.voice = voice;
        return this;
    }

    function Channel(channel)
    {
        this.channel = channel;
        return this;
    }

    function Params(params)
    {
        this.paramsOverride = params;
        return this;
    }

    function Volume(volume)
    {
        this.volume = volume;
        return this;
    }

    function Double(double = true)
    {
        this.double = double;
        return this;
    }

    function SinglePlayer(singlePlayer = true)
    {
        this.singlePlayer = singlePlayer;
        return this;
    }

    function Play()
    {
        if (delay == null)
            PlayInternal();
        else
            RunWithDelay(delay, function(soundBuilder) {
                soundBuilder.PlayInternal();
            }, this, scope);
    }

    function PlayInternal()
    {
        local randomSound = RandomElement(safeget(safeget(scope, "soundBank", null), sound, []));
        if (!randomSound)
            return false;

        isGlobal = isGlobal || (!entity && !safeget(scope, "self", null));
        if (positional == null)
            positional = !isGlobal;

        local params = {
            sound_name = randomSound,
            volume = volume,
            filter_type = singlePlayer ? RECIPIENT_FILTER_SINGLE_PLAYER : isGlobal ? RECIPIENT_FILTER_GLOBAL : 0,
            sound_level = positional ? clampCeiling(150, soundLevel) : 0
            channel = channel ? channel : voice
                ? safeget(scope, "mute", false) ? 141 : 2
                : 6
        };

        if (entity)
        {
            params.entity <- entity;
            params.speaker_entity <- entity;
        }

        if (paramsOverride != null)
            params = combinetables(params, paramsOverride);
        EmitSoundEx(params);

        if (double || (positional && isGlobal))
        {
            local extraSpeaker = safeget(scope, "extraSoundBankSpeaker", null);
            EmitSoundEx(combinetables(params, {
            	entity = extraSpeaker,
                speaker_entity = extraSpeaker
            }));
        }
        return true;
    }
}
::PlaySoundBuilder <- PlaySoundBuilderClass

::BuildSound <- function(sound, scope = null)
{
    if (scope == null)
        scope = this;
    return PlaySoundBuilder(sound).Scope(scope);
}

::PlayVoiceLineForAnotherPlayer <- function(listener, sound, params = {})
{
    local randomSound = RandomElement(safeget(soundBank, sound, []));
    if (!randomSound)
        return false;
    //Thanks ficool2 for the idea
    local offset = Vector(999999, 999999, 999999);
    local restore = {};
    foreach (player in GetPlayers())
        if (player != listener)
        {
            restore[player] <- GetPropVector(player, "m_vecViewOffset");
            SetPropVector(player, "m_vecViewOffset", offset);
        }

    for (local i = 0; i < 2; i++)
        EmitSoundEx(combinetables({
            sound_name = randomSound,
            filter_type = RECIPIENT_FILTER_PAS_ATTENUATION,
            channel = !i ? 6 : mute ? 141 : 2,
            entity = self,
            speaker_entity = self,
            volume = 1,
            flags =  1,
            sound_level = 150
        }, params));

    foreach (player, offset in restore)
        SetPropVector(player, "m_vecViewOffset", offset);

    return randomSound == null;
}