local lastWeaponEnt = null;
local playerInTaunt = false;

AddTimer(-1, -10, function()
{
    local inCond = self.InCond(TF_COND_TAUNTING);
    if (inCond && !playerInTaunt)
    {
        playerInTaunt = true;
    }
    else if (!inCond && playerInTaunt)
    {
        playerInTaunt = false;
        FixViewmodels();
        self.Weapon_Switch(self.GetActiveWeapon());
    }

    local weaponEnt = self.GetActiveWeapon();

    if (weaponEnt == lastWeaponEnt)
        return;
    if (!IsValid(lastWeaponEnt))
        lastWeaponEnt = null;
    SetPropEntity(self, "m_hLastWeapon", lastWeaponEnt);

    local lastWeaponTT = safescriptget(lastWeaponEnt, "weaponTT", null);
    local weaponTT = safescriptget(weaponEnt, "weaponTT", null);

    safescriptcall(lastWeaponEnt, "OnSwitchOff", [self, lastWeaponTT, lastWeaponEnt, weaponTT], null);
    safescriptcall(weaponEnt, "OnSwitch", [self, weaponTT, weaponEnt, lastWeaponTT], null);

    lastWeaponEnt = weaponEnt;
});

function FixViewmodels()
{
    local weaponEnt, weaponTT;
	for (local i = 0; i < 27; i++)
        if ((weaponEnt = GetPropEntityArray(self, "m_hMyWeapons", i))
            && (weaponTT = safeget(weaponEnt.GetScriptScope(), "weaponTT", null)))
                FixViewmodel(weaponEnt, weaponTT)
}

function FixViewmodel(weaponEnt, weaponTT)
{
    if ("viewmodel" in weaponTT)
    {
        local viewmodelIndex = GetModelIndex(weaponTT.viewmodel);

        weaponEnt.SetModel(weaponTT.viewmodel);
        weaponEnt.SetCustomViewModelModelIndex(viewmodelIndex);
        SetPropInt(weaponEnt, "m_nModelIndex", viewmodelIndex);
        SetPropInt(weaponEnt, "m_iViewModelIndex", viewmodelIndex);
    }
    if ("worldmodel" in weaponTT)
    {
        SetPropBool(weaponEnt, "m_bBeingRepurposedForTaunt", true);
        PrecacheModel(weaponTT.worldmodel);

        if (!safeget(weaponTT, "worldmodelfix", 0))
        {
            ChangeWorldModel(weaponEnt, weaponTT.worldmodel);
        }
        else
        {
            weaponEnt.DisableDraw();
            SetPropInt(weaponEnt, "m_nRenderMode", 1);
            weaponEnt.SetModelScale(0.05, 0);
            SetEntityColor(weaponEnt, [0, 0, 0, 0])
            EntFireByHandle(weaponEnt, "DisableShadow", "", -1, null, null);
        }
    }
}

function CreateWeapon(weaponTT)
{
    local weaponEnt = Entities.CreateByClassname(weaponTT.baseclass);
    SetPropInt(weaponEnt, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", weaponTT.baseid);
    SetPropBool(weaponEnt, "m_AttributeManager.m_Item.m_bInitialized", true);
    SetPropBool(weaponEnt, "m_bValidatedAttachedEntity", true);
	SetPropBool(weaponEnt, "m_bClientSideAnimation", true);
    weaponEnt.SetTeam(self.GetTeam());
    weaponEnt.SetOwner(self);
	SetPropEntity(weaponEnt, "m_hOwnerEntity", self);
    SetPropBool(weaponEnt, "m_bForcePurgeFixedupStrings", true);
    Entities.DispatchSpawn(weaponEnt);
    return weaponEnt;
}

function EquipWeapon(weaponTT)
{
    local weaponEnt = CreateWeapon(weaponTT);
    self.Weapon_Equip(weaponEnt);

    weaponEnt.ValidateScriptScope();
    local weaponScript = weaponEnt.GetScriptScope();
    weaponScript.weaponTT <- weaponTT;

    FixViewmodel(weaponEnt, weaponTT);

    if ("OnEquip" in weaponTT)
    {
        weaponScript.OnEquip <- weaponTT.OnEquip;
        weaponScript.OnEquip(self, weaponTT, weaponEnt);
    }
    if ("OnSwitch" in weaponTT)
        weaponScript.OnSwitch <- weaponTT.OnSwitch;
    if ("OnSwitchOff" in weaponTT)
        weaponScript.OnSwitchOff <- weaponTT.OnSwitchOff;
    if ("OnFire" in weaponTT)
        Include("/_charlib/weapon_events.nut", weaponScript)

    return weaponEnt;
}